"""Plot module for coolpandas."""
from .barplot import barplot
from .boxplot import boxplot
from .distplot import distplot
from .geoplot import geoplot
from .lineplot import lineplot
from .mapplot import mapplot
